@@ -1,14 +1,16 @@
+import collections
+
 from six import iteritems
 from itertools import chain
 
 from .utils import DslBase
-from .field import InnerObject, String
+from .field import InnerObject, Text
 from .connections import connections
 from .exceptions import IllegalOperation
 
 META_FIELDS = frozenset((
     'dynamic', 'transform', 'dynamic_date_formats', 'date_detection',
-    'numeric_detection', 'dynamic_templates'
+    'numeric_detection', 'dynamic_templates', 'enabled'
 ))
 
 class Properties(InnerObject, DslBase):
@@ -38,14 +40,23 @@ class Mapping(object):
         m.update_from_es(index, using)
         return m
 
+    def resolve_field(self, field_path):
+        field = self
+        for step in field_path.split('.'):
+            try:
+                field = field[step]
+            except KeyError:
+                return
+        return field
+
     def _collect_analysis(self):
         analysis = {}
         fields = []
         if '_all' in self._meta:
-            fields.append(String(**self._meta['_all']))
+            fields.append(Text(**self._meta['_all']))
 
         for f in chain(fields, self.properties._collect_fields()):
-            for analyzer_name in ('analyzer', 'search_analyzer'):
+            for analyzer_name in ('analyzer', 'search_analyzer', 'search_quote_analyzer'):
                 if not hasattr(f, analyzer_name):
                     continue
                 analyzer = getattr(f, analyzer_name)
@@ -88,7 +99,7 @@ class Mapping(object):
         # metadata like _all etc
         for name, value in iteritems(raw):
             if name != 'properties':
-                if isinstance(value, dict):
+                if isinstance(value, collections.Mapping):
                     self.meta(name, **value)
                 else:
                     self.meta(name, value)
@@ -144,7 +155,7 @@ class Mapping(object):
         if '_all' in meta:
             meta = meta.copy()
             _all = meta['_all'] = meta['_all'].copy()
-            for f in ('analyzer', 'search_analyzer'):
+            for f in ('analyzer', 'search_analyzer', 'search_quote_analyzer'):
                 if hasattr(_all.get(f, None), 'to_dict'):
                     _all[f] = _all[f].to_dict()
         d[self.doc_type].update(meta)
