@@ -1,3 +1,6 @@
+import copy
+import collections
+
 from six import iteritems, string_types
 
 from elasticsearch.helpers import scan
@@ -6,7 +9,7 @@ from elasticsearch.exceptions import TransportError
 from .query import Q, EMPTY_QUERY, Bool
 from .aggs import A, AggBase
 from .utils import DslBase
-from .result import Response, Result, SuggestResponse
+from .response import Response, Hit, SuggestResponse
 from .connections import connections
 
 class QueryProxy(object):
@@ -26,7 +29,7 @@ class QueryProxy(object):
 
     def __call__(self, *args, **kwargs):
         s = self._search._clone()
-        getattr(s, self._attr_name)._proxied += Q(*args, **kwargs)
+        getattr(s, self._attr_name)._proxied &= Q(*args, **kwargs)
 
         # always return search to be chainable
         return s
@@ -84,7 +87,7 @@ class Request(object):
         if isinstance(doc_type, (tuple, list)):
             for dt in doc_type:
                 self._add_doc_type(dt)
-        elif isinstance(doc_type, dict):
+        elif isinstance(doc_type, collections.Mapping):
             self._doc_type.extend(doc_type.keys())
             self._doc_type_map.update(doc_type)
         elif doc_type:
@@ -128,7 +131,7 @@ class Request(object):
 
     def _add_doc_type(self, doc_type):
         if hasattr(doc_type, '_doc_type'):
-            self._doc_type_map[doc_type._doc_type.name] = doc_type.from_es
+            self._doc_type_map[doc_type._doc_type.name] = doc_type
             doc_type = doc_type._doc_type.name
         self._doc_type.append(doc_type)
 
@@ -138,7 +141,7 @@ class Request(object):
         multiple. Values can be strings or subclasses of ``DocType``.
 
         You can also pass in any keyword arguments, mapping a doc_type to a
-        callback that should be used instead of the Result class.
+        callback that should be used instead of the Hit class.
 
         If no doc_type is supplied any information stored on the instance will
         be erased.
@@ -212,8 +215,6 @@ class Search(Request):
         self.aggs = AggsProxy(self)
         self._sort = []
         self._source = None
-        self._fields = None
-        self._partial_fields = {}
         self._highlight = {}
         self._highlight_opts = {}
         self._suggest = {}
@@ -296,9 +297,8 @@ class Search(Request):
 
         s._response_class = self._response_class
         s._sort = self._sort[:]
-        s._source = self._source.copy() if self._source else None
-        s._fields = self._fields[:] if self._fields else None
-        s._partial_fields = self._partial_fields.copy()
+        s._source = copy.copy(self._source) \
+            if self._source is not None else None
         s._highlight = self._highlight.copy()
         s._highlight_opts = self._highlight_opts.copy()
         s._suggest = self._suggest.copy()
@@ -340,10 +340,6 @@ class Search(Request):
             self._sort = d.pop('sort')
         if '_source' in d:
             self._source = d.pop('_source')
-        if 'fields' in d:
-            self._fields = d.pop('fields')
-        if 'partial_fields' in d:
-            self._partial_fields = d.pop('partial_fields')
         if 'highlight' in d:
             high = d.pop('highlight').copy()
             self._highlight = high.pop('fields')
@@ -384,7 +380,7 @@ class Search(Request):
         s._script_fields.update(kwargs)
         return s
 
-    def source(self, **kwargs):
+    def source(self, fields=None, **kwargs):
         """
         Selectively control how the _source field is returned.
 
@@ -408,7 +404,14 @@ class Search(Request):
         """
         s = self._clone()
 
-        if s._source is None:
+        if fields and kwargs:
+            raise ValueError("You cannot specify fields and kwargs at the same time.")
+
+        if fields is not None:
+            s._source = fields
+            return s
+
+        if kwargs and not isinstance(s._source, dict):
             s._source = {}
 
         for key, value in kwargs.items():
@@ -422,43 +425,6 @@ class Search(Request):
 
         return s
 
-    def fields(self, fields=None):
-        """
-        Selectively load specific stored fields for each document.
-
-        :arg fields: list of fields to return for each document
-
-        If ``fields`` is None, the entire document will be returned for
-        each hit.  If fields is the empty list, no fields will be
-        returned for each hit, just the metadata.
-        """
-        s = self._clone()
-        s._fields = fields
-        return s
-
-    def partial_fields(self, **partial):
-        """
-        Control which part of the fields to extract from the `_source` document
-
-        :kwargs partial: dict specifying which fields to extract from the source
-
-        An example usage would be:
-
-            s = Search().partial_fields(authors_data={
-                    'include': ['authors.*'],
-                    'exclude': ['authors.name']
-                })
-
-        which will include all fields from the `authors` nested property except for
-        each authors `name`
-
-        If ``partial`` is not provided, the whole `_source` will be fetched. Calling this multiple
-        times will override the previous values with the new ones.
-        """
-        s = self._clone()
-        s._partial_fields = partial
-        return s
-
     def sort(self, *keys):
         """
         Add sorting information to the search request. If called without
@@ -555,11 +521,11 @@ class Search(Request):
         """
         d = {"query": self.query.to_dict()}
 
-        if self.post_filter:
-            d['post_filter'] = self.post_filter.to_dict()
-
         # count request doesn't care for sorting and other things
         if not count:
+            if self.post_filter:
+                d['post_filter'] = self.post_filter.to_dict()
+
             if self.aggs.aggs:
                 d.update(self.aggs.to_dict())
 
@@ -568,15 +534,9 @@ class Search(Request):
 
             d.update(self._extra)
 
-            if self._source:
+            if not self._source in (None, {}):
                 d['_source'] = self._source
 
-            if self._fields is not None:
-                d['fields'] = self._fields
-
-            if self._partial_fields:
-                d['partial_fields'] = self._partial_fields
-
             if self._highlight:
                 d['highlight'] = {'fields': self._highlight}
                 d['highlight'].update(self._highlight_opts)
@@ -620,13 +580,13 @@ class Search(Request):
             es = connections.get_connection(self._using)
 
             self._response = self._response_class(
+                self,
                 es.search(
                     index=self._index,
                     doc_type=self._doc_type,
                     body=self.to_dict(),
                     **self._params
-                ),
-                callbacks=self._doc_type_map
+                )
             )
         return self._response
 
@@ -663,10 +623,16 @@ class Search(Request):
                 doc_type=self._doc_type,
                 **self._params
             ):
-            yield self._doc_type_map.get(hit['_type'], Result)(hit)
+            callback = self._doc_type_map.get(hit['_type'], Hit)
+            callback = getattr(callback, 'from_es', callback)
+            yield callback(hit)
 
 
 class MultiSearch(Request):
+    """
+    Combine multiple :class:`~elasticsearch_dsl.Search` objects into a single
+    request.
+    """
     def __init__(self, **kwargs):
         super(MultiSearch, self).__init__(**kwargs)
         self._searches = []
@@ -683,6 +649,13 @@ class MultiSearch(Request):
         return ms
 
     def add(self, search):
+        """
+        Adds a new :class:`~elasticsearch_dsl.Search` object to the request::
+
+            ms = MultiSearch(index='my-index')
+            ms = ms.add(Search(doc_type=Category).filter('term', category='python'))
+            ms = ms.add(Search(doc_type=Blog))
+        """
         ms = self._clone()
         ms._searches.append(search)
         return ms
@@ -703,6 +676,9 @@ class MultiSearch(Request):
         return out
 
     def execute(self, ignore_cache=False, raise_on_error=True):
+        """
+        Execute the multi search request and return a list of search results.
+        """
         if ignore_cache or not hasattr(self, '_response'):
             es = connections.get_connection(self._using)
 
@@ -716,13 +692,11 @@ class MultiSearch(Request):
             out = []
             for s, r in zip(self._searches, responses['responses']):
                 if r.get('error', False):
-                    print(r)
                     if raise_on_error:
                         raise TransportError('N/A', r['error']['type'], r['error'])
                     r = None
                 else:
-                    r = Response(r, callbacks=s._doc_type_map)
-                    r.search = s
+                    r = Response(s, r)
                 out.append(r)
 
             self._response = out
